@@ -44,6 +44,7 @@
 #if COMPRESSION_ZSTD
 #include <zstd.h>
 #endif
+#include "kernel-shared/uapi/btrfs.h"
 #include "kernel-shared/ctree.h"
 #include "common/defs.h"
 #include "common/messages.h"
@@ -55,7 +56,6 @@
 #include "common/string-utils.h"
 #include "cmds/commands.h"
 #include "cmds/receive-dump.h"
-#include "ioctl.h"
 
 struct btrfs_receive
 {
@@ -297,7 +297,8 @@ static int process_snapshot(const char *path, const u8 *uuid, u64 ctransid,
 			ret = -ENOENT;
 		else
 			ret = PTR_ERR(parent_subvol);
-		error("cannot find parent subvolume");
+		uuid_unparse(parent_uuid, uuid_str);
+		error("snapshot: cannot find parent subvolume %s", uuid_str);
 		goto out;
 	}
 
@@ -749,11 +750,14 @@ static int process_clone(const char *path, u64 offset, u64 len,
 					NULL,
 					subvol_search_by_received_uuid);
 		if (IS_ERR_OR_NULL(si)) {
+			char uuid_str[BTRFS_UUID_UNPARSED_SIZE];
+
 			if (!si)
 				ret = -ENOENT;
 			else
 				ret = PTR_ERR(si);
-			error("clone: did not find source subvol");
+			uuid_unparse(clone_uuid, uuid_str);
+			error("clone: cannot find source subvol %s", uuid_str);
 			goto out;
 		}
 		/* strip the subvolume that we are receiving to from the start of subvol_path */
@@ -1080,6 +1084,15 @@ static int decompress_zstd(struct btrfs_receive *rctx, const char *encoded_buf,
 			return -EIO;
 		}
 	}
+
+	/*
+	 * Zero out the unused part of the output buffer.
+	 * At least with zstd 1.5.2, the decompression can leave some garbage
+	 * at/beyond the offset out_buf.pos.
+	 */
+	if (out_buf.pos < out_buf.size)
+		memset(unencoded_buf + out_buf.pos, 0, out_buf.size - out_buf.pos);
+
 	return 0;
 }
 #endif
@@ -1619,25 +1632,23 @@ static const char * const cmd_receive_usage[] = {
 	"After receiving a subvolume, it is immediately set to",
 	"read-only.",
 	"",
-	"-q|--quiet       suppress all messages, except errors",
-	"-f FILE          read the stream from FILE instead of stdin",
-	"-e               terminate after receiving an <end cmd> marker in the stream.",
-	"                 Without this option the receiver side terminates only in case",
-	"                 of an error on end of file.",
-	"-C|--chroot      confine the process to <mount> using chroot",
-	"-E|--max-errors NERR",
-	"                 terminate as soon as NERR errors occur while",
-	"                 stream processing commands from the stream.",
-	"                 Default value is 1. A value of 0 means no limit.",
-	"-m ROOTMOUNT     the root mount point of the destination filesystem.",
-	"                 If /proc is not accessible, use this to tell us where",
-	"                 this file system is mounted.",
-	"--force-decompress",
-	"                 if the stream contains compressed data, always",
-	"                 decompress it instead of writing it with encoded I/O",
-	"--dump           dump stream metadata, one line per operation,",
-	"                 does not require the MOUNT parameter",
-	"-v               deprecated, alias for global -v option",
+	OPTLINE("-q|--quiet", "suppress all messages, except errors"),
+	OPTLINE("-f FILE", "read the stream from FILE instead of stdin"),
+	OPTLINE("-e", "terminate after receiving an <end cmd> marker in the stream. "
+		"Without this option the receiver side terminates only in case "
+		"of an error on end of file."),
+	OPTLINE("-C|--chroot", "confine the process to <mount> using chroot"),
+	OPTLINE("-E|--max-errors NERR", "terminate as soon as NERR errors occur while "
+		"stream processing commands from the stream. "
+		"Default value is 1. A value of 0 means no limit."),
+	OPTLINE("-m ROOTMOUNT", "the root mount point of the destination filesystem. "
+		"If /proc is not accessible, use this to tell us where "
+		"this file system is mounted."),
+	OPTLINE("--force-decompress", "if the stream contains compressed data, always "
+		"decompress it instead of writing it with encoded I/O"),
+	OPTLINE("--dump", "dump stream metadata, one line per operation, "
+		"does not require the MOUNT parameter"),
+	OPTLINE("-v", "deprecated, alias for global -v option"),
 	HELPINFO_INSERT_GLOBALS,
 	HELPINFO_INSERT_VERBOSE,
 	HELPINFO_INSERT_QUIET,
@@ -1753,9 +1764,9 @@ static int cmd_receive(const struct cmd_struct *cmd, int argc, char **argv)
 	}
 
 	if (dump && check_argc_exact(argc - optind, 0))
-		usage(cmd);
+		usage(cmd, 1);
 	if (!dump && check_argc_exact(argc - optind, 1))
-		usage(cmd);
+		usage(cmd, 1);
 
 	tomnt = argv[optind];
 
