@@ -2,7 +2,7 @@
 #
 # test file name length limits for subvolumes
 
-source "$TEST_TOP/common"
+source "$TEST_TOP/common" || exit
 
 check_prereq mkfs.btrfs
 check_prereq btrfs
